<?php

namespace FiloBlu\AdyenErrorNotificationHandler\Observer;

use FiloBlu\AdyenErrorNotificationHandler\Helper\Data;
use Magento\Framework\DataObject;

class ErrorNotify implements \Magento\Framework\Event\ObserverInterface
{
    /**
     * @var Data
     */
    protected $dataHelper;

    public function __construct(Data $dataHelper)
    {
        $this->dataHelper = $dataHelper;
    }

    /**
     * Execute observer
     *
     * @param \Magento\Framework\Event\Observer $observer
     * @return void
     */
    public function execute(
        \Magento\Framework\Event\Observer $observer
    )
    {
        if ($this->dataHelper->adyenErrorNotificationHandlerIsEnabled()) {
            $response = $observer->getEvent()->getResponse();
            $language = $observer->getEvent()->getLanguage();
            $is3dsV1 = $observer->getEvent()->getSecureCodeOne();
            $error = $observer->getEvent()->getDefaultErrorMessage();

            $errorMessages = json_decode($this->dataHelper->getAdyenErrorNotificationMessages(), true);

            if (!is_null($errorMessages) && is_array($errorMessages) && isset($response[$this->dataHelper::ADYEN_REFUSAL_REASON_CODE])) {
                $errorInLang = $this->dataHelper->getErrorsInLanguage($errorMessages, $language);

                if (is_null($errorInLang) && $this->dataHelper->useErrorMessagesInDefaultLangEnabled()) {
                    /*
                     * If the messages translations are missing, I search the default language translations.
                     */
                    $defaultLang = $this->dataHelper->getDefaultLang();
                    $errorInLang = $this->dataHelper->getErrorsInLanguage($errorMessages, $defaultLang);
                }

                if (!is_null($errorInLang)) {
                    $errorArray = array();
                    $error = $this->dataHelper->getErrorByCode($errorInLang, $response[$this->dataHelper::ADYEN_REFUSAL_REASON_CODE]);
                    if (!is_null($error)) {
                        /**
                         * @var DataObject
                         */
                        $checkMappedMessage = $observer->getEvent()->getMappedErrorMessage();
                        $errorArray[] = $error;
                        $checkMappedMessage->setData('mapped_error', $errorArray);
                    }
                }
            }

            $dataToLog = array(
                'response' => $response,
                'lang' => $language,
                '3ds_v1' => $is3dsV1,
                'module_error' => $error
            );
            $this->dataHelper->manageDataToLog($dataToLog);
        }
    }
}